### Sequence-capture phylogenomics of early-diverging true spiders reveals convergent evolution of tracheal systems after the origin of aerial webs
### Martín J. Ramírez, Ivan L.F. Magalhaes, Shahan Derkarabetian, Joel Ledford, Charles E. Griswold, Hannah M. Wood, Marshal Hedin

###reconstruct ancestral states for posterior respiratory system morphology in spiders using different models
###tests fit of different models using AICs & Akaike weights, and LRTs

### coding scheme: PRS coded as "booklungs" or "tracheae", absences coded as missing ("NA")

#sets working directory and reads inputs, prunes tree to include only terminals listed in table with observations

require(phytools)
require(phytools)
require(ape)

setwd("C:/Users/Usuario/Google Drive/Basal Araneomorphae/analyses morphology - Early Araneomoprhae/phytools/EDaraneomorphs/1b-booklungs-tracheae-NA")
sink("output.txt", append=FALSE, split=TRUE) 

tiplabels<-read.csv("respsystem-original.csv",row.names=1)
resp.syst.tips<-setNames(tiplabels[,1],rownames(tiplabels))
tipcols<-setNames(c("black","red","blue"),levels(resp.syst.tips))

scorings<-read.csv("respsystem1.csv",row.names=1)
species<-rownames(scorings)
resp.syst<-setNames(scorings[,1],rownames(scorings))
resp.syst
unpruned_araneom_tree<-read.tree("araneomorphs.tre")
araneom_tree<-keep.tip(unpruned_araneom_tree, species)

#########################
### EQUAL RATES MODEL ###
#########################

#uses a single-rate model to reconstruct states at internal nodes; plots and reports rates
fitER<-ace(resp.syst,araneom_tree,model="ER",type="discrete")

#plots tree with pie charts with likelihood of ancestral states at nodes using the single rate model
pdf("ER-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red", "blue"),levels(resp.syst))
tiplabels(pie=to.matrix(resp.syst.tips[araneom_tree$tip.label], levels(resp.syst.tips)),piecol=tipcols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=fitER$lik.anc,piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off() 

##########################
### IRREVERSIBLE MODEL ###
##########################

#uses a model that disallows reversals from tracheae to booklungs, or regains of respiratory structures
irreversible<-model<-matrix(c(0,1,0,0),2,2,byrow=TRUE, dimnames=list(levels(resp.syst),levels(resp.syst)))
fitIRV<-ace(resp.syst,araneom_tree,model=irreversible,type="discrete")

#plots tree with pie charts with likelihood of ancestral states at nodes using the multiple rates model
pdf("IRV-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red", "blue"),levels(resp.syst))
tiplabels(pie=to.matrix(resp.syst.tips[araneom_tree$tip.label], levels(resp.syst.tips)),piecol=tipcols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=fitIRV$lik.anc,piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off() 

#################################
### ALL RATES DIFFERENT MODEL ###
#################################

#uses a model allowing for different rates between states to reconstruct states at internal nodes
fitARD<-ace(resp.syst,araneom_tree,model="ARD",type="discrete")

#plots tree with pie charts with likelihood of ancestral states at nodes using the multiple rates model
pdf("ARD-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red", "blue"),levels(resp.syst))
tiplabels(pie=to.matrix(resp.syst.tips[araneom_tree$tip.label], levels(resp.syst.tips)),piecol=tipcols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=fitARD$lik.anc,piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off() 

#######################
### TESTS MODEL FIT ###
#######################

#reports general results
sink(file = NULL)
sink("model_comparison.txt", append=FALSE, split=TRUE)
print("model results:")
modelresults<-matrix(c(logLik(fitER),logLik(fitIRV),logLik(fitARD),length(fitER$rates),length(fitIRV$rates),length(fitARD$rates)), nrow=2, ncol=3, byrow=TRUE, dimnames=list(c("log likelihood","number of parameters"),c("ER","IRV","ARD")))
modelresults
print("rates ER:")
fitER$rates
print("rates IRV:")
fitIRV$rates
print("rates ARD:")
fitARD$rates

#calculates fit of models using AIC...
AIC<-setNames(sapply(list(fitER,fitIRV,fitARD),AIC),c("ER","IRV","ARD"))
print("AIC:")
AIC
print(" ")
print("AIC weights:")
aic.w(AIC)

#compares models using LRT...
print(" ")
print("LRT results:")
print("ER x IRV")
anova(fitER,fitIRV)
print("ER x ARD")
anova(fitER,fitARD)
print("IRV x ARD")
anova(fitIRV,fitARD)
sink(file = NULL)