### Sequence-capture phylogenomics of early-diverging true spiders reveals convergent evolution of tracheal systems after the origin of aerial webs
### Martín J. Ramírez, Ivan L.F. Magalhaes, Shahan Derkarabetian, Joel Ledford, Charles E. Griswold, Hannah M. Wood, Marshal Hedin

###reconstruct ancestral states for posterior respiratory system morphology in spiders using different models
###tests fit of different models using AICs & Akaike weights

### coding scheme: PRS coded as "booklungs", "lamellae", "single tube and leaf", "tubular tracheae", terminals without PRS pruned from the analysis

#sets working directory and reads inputs, prunes tree to include only terminals listed in table with observations

require(phytools)
require(ape)

setwd("C:/Users/Usuario/Google Drive/Basal Araneomorphae/analyses morphology - Early Araneomoprhae/phytools/EDaraneomorphs/3b-finely-coded-pruned-noabsence")
sink("output.txt", append=FALSE, split=TRUE) 

scorings<-read.csv("respsystem2.csv",row.names=1)
species<-rownames(scorings)
resp.syst<-setNames(scorings[,1],rownames(scorings))
resp.syst
unpruned_araneom_tree<-read.tree("araneomorphs.tre")
araneom_tree<-keep.tip(unpruned_araneom_tree, species)

#########################
### EQUAL RATES MODEL ###
#########################

#uses a single-rate model to reconstruct states at internal nodes; plots and reports rates
fitER<-ace(resp.syst,araneom_tree,model="ER",type="discrete")
ratesER<-fitMk(araneom_tree,resp.syst,model="ER")
pdf("ER-rates.pdf")
plot(ratesER)
dev.off() 
print("equal rates model:")
ratesER

#plots tree with pie charts with likelihood of ancestral states at nodes using the single rate model
pdf("ER-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red","yellow", "green", "blue"),levels(resp.syst))
tiplabels(pie=to.matrix(resp.syst[araneom_tree$tip.label], levels(resp.syst)),piecol=cols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=fitER$lik.anc,piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off() 

#############################
### SYMMETRIC RATES MODEL ###
#############################

#uses a model with symmetric rates between states to reconstruct states at internal nodes
fitSYM<-ace(resp.syst,araneom_tree,model="SYM",type="discrete")
ratesSYM<-fitMk(araneom_tree,resp.syst,model="SYM")
pdf("SYM-rates.pdf")
plot(ratesSYM)
dev.off() 
print("symmetric model:")
ratesSYM

#plots tree with pie charts with likelihood of ancestral states at nodes using the multiple rates model
pdf("SYM-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red","yellow", "green", "blue"),levels(resp.syst))
tiplabels(pie=to.matrix(resp.syst[araneom_tree$tip.label], levels(resp.syst)),piecol=cols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=fitSYM$lik.anc,piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off() 

##########################
### IRREVERSIBLE MODEL ###
##########################

#uses a model that disallows reversals from tracheae to booklungs, and transitions among tracheae
irreversible<-matrix(c(0,1,2,3,0,0,0,0,0,0,0,0,0,0,0,0),4,4,byrow=TRUE, dimnames=list(levels(resp.syst),levels(resp.syst)))
fitIRV<-ace(resp.syst,araneom_tree,model=irreversible,type="discrete")
ratesIRV<-fitMk(araneom_tree,resp.syst,model=irreversible)
pdf("IRV-rates.pdf")
plot(ratesIRV)
dev.off() 
print("irreversible model:")
ratesIRV

#plots tree with pie charts with likelihood of ancestral states at nodes using the multiple rates model
pdf("IRV-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red","yellow", "green", "blue"),levels(resp.syst))
tiplabels(pie=to.matrix(resp.syst[araneom_tree$tip.label], levels(resp.syst)),piecol=cols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=abs(fitIRV$lik.anc),piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off()

############################
### IRREVERSIBLE MODEL 2 ###
############################

#uses a model that disallows reversals from tracheae to booklungs
irreversible2<-matrix(c(0,1,2,3,0,0,4,5,0,6,0,7,0,8,9,0),4,4,byrow=TRUE, dimnames=list(levels(resp.syst),levels(resp.syst)))
fitIRV2<-ace(resp.syst,araneom_tree,model=irreversible2,type="discrete")
ratesIRV2<-fitMk(araneom_tree,resp.syst,model=irreversible2)
pdf("IRV2-rates.pdf")
plot(ratesIRV2)
dev.off() 
print("irreversible model 2:")
ratesIRV2

#plots tree with pie charts with likelihood of ancestral states at nodes using the multiple rates model
pdf("IRV2-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red","yellow", "green", "blue"),levels(resp.syst))
tiplabels(pie=to.matrix(resp.syst[araneom_tree$tip.label], levels(resp.syst)),piecol=cols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=abs(fitIRV2$lik.anc),piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off()

#################################
### ALL RATES DIFFERENT MODEL ###
#################################

#uses a model allowing for different rates between states to reconstruct states at internal nodes
fitARD<-ace(resp.syst,araneom_tree,model="ARD",type="discrete")
ratesARD<-fitMk(araneom_tree,resp.syst,model="ARD")
pdf("ARD-rates.pdf")
plot(ratesARD)
dev.off() 
print("all rates different model:")
ratesARD

#plots tree with pie charts with likelihood of ancestral states at nodes using the multiple rates model
pdf("ARD-tree.pdf")
plotTree(araneom_tree,type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red","yellow", "green", "blue"),levels(resp.syst))
tiplabels(pie=to.matrix(resp.syst[araneom_tree$tip.label], levels(resp.syst)),piecol=cols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=fitARD$lik.anc,piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off() 

#######################
### TESTS MODEL FIT ###
#######################

#calculates fit of models using AIC...
AIC<-setNames(sapply(list(ratesER,ratesSYM,ratesIRV,ratesIRV2,ratesARD),AIC),c("ER","SYM","IRV", "IRV2","ARD"))
sink(file = NULL)
sink("model_comparison.txt", append=FALSE, split=TRUE) 
print("AIC:")
AIC
print(" ")
print("AIC weights:")
aic.w(AIC)
print(" ")

#...or a LRT
kER<-length(ratesER$rates)
kSYM<-length(ratesSYM$rates)
kIRV<-length(ratesIRV$rates)
kIRV2<-length(ratesIRV2$rates)
kARD<-length(ratesARD$rates)

modelresults<-matrix(c(logLik(ratesER),logLik(ratesSYM),logLik(ratesIRV),logLik(ratesIRV2),logLik(ratesARD),kER,kSYM,kIRV,kIRV2,kARD), nrow=2, ncol=5, byrow=TRUE, dimnames=list(c("log likelihood","number of parameters"),c("ER","SYM","IRV","IRV2","ARD")))
modelresults

LR_ERxSYM<--2*(logLik(ratesER)-logLik(ratesSYM))
LR_ERxARD<--2*(logLik(ratesER)-logLik(ratesARD))
LR_SYMxARD<--2*(logLik(ratesSYM)-logLik(ratesARD))
LR_IRVxIRV2<--2*(logLik(ratesIRV)-logLik(ratesIRV2))
LR_IRVxARD<--2*(logLik(ratesIRV)-logLik(ratesARD))
LR_IRV2xARD<--2*(logLik(ratesIRV2)-logLik(ratesARD))

P_chisq_ERxSYM<-pchisq(LR_ERxSYM,df=kSYM-kER)
P_chisq_ERxARD<-pchisq(LR_ERxARD,df=kARD-kER)
P_chisq_SYMxARD<-pchisq(LR_SYMxARD,df=kARD-kSYM)
P_chisq_IRVxIRV2<-pchisq(LR_IRVxIRV2,df=kIRV2-kIRV)
P_chisq_IRVxARD<-pchisq(LR_IRVxARD,df=kARD-kIRV)
P_chisq_IRV2xARD<-pchisq(LR_IRV2xARD,df=kARD-kIRV2)

LRTresults<-matrix(c(LR_ERxSYM, LR_ERxARD, LR_SYMxARD, LR_IRVxIRV2, LR_IRVxARD, LR_IRV2xARD, P_chisq_ERxSYM, P_chisq_ERxARD, P_chisq_SYMxARD, P_chisq_IRVxIRV2, P_chisq_IRVxARD, P_chisq_IRV2xARD), nrow=2, ncol=6, byrow=TRUE, dimnames=list(c("LR","p"),c("ERxSYM", "ERxARD", "SYMxARD", "IRVxIRV2","IRVxARD","IRV2xARD")))
print(" ")
print("LRT results:")
LRTresults
sink(file = NULL)