### Sequence-capture phylogenomics of early-diverging true spiders reveals convergent evolution of tracheal systems after the origin of aerial webs
### Martín J. Ramírez, Ivan L.F. Magalhaes, Shahan Derkarabetian, Joel Ledford, Charles E. Griswold, Hannah M. Wood, Marshal Hedin

###reconstruct ancestral states for posterior respiratory system morphology in spiders using different rate models
###tests fit of different models using AICs & Akaike weights, and LRTs

### coding scheme: PRS coded as "booklungs", "lamellae", "single tube and leaf", "tubular tracheae", absences coded as missing ("NA")

#sets working directory and reads inputs, prunes tree to include only terminals listed in table with observations

require(phytools)
require(ape)

setwd("C:/Users/Usuario/Google Drive/Basal Araneomorphae/analyses morphology - Early Araneomoprhae/phytools/EDaraneomorphs/3c-finely-coded-pruned-NA-v2")
sink("output.txt", append=FALSE, split=TRUE) 

tiplabels<-read.csv("respsystem-original.csv",row.names=1)
resp.syst.tips<-setNames(tiplabels[,1],rownames(tiplabels))
tipcols<-setNames(c("black","red","yellow", "green", "blue"),levels(resp.syst.tips))

scorings<-read.csv("respsystem2.csv",row.names=1)
species<-rownames(scorings)
resp.syst<-setNames(scorings[,1],rownames(scorings))
resp.syst
unpruned_araneom_tree<-read.tree("araneomorphs.tre")
araneom_tree<-keep.tip(unpruned_araneom_tree, species)

#########################
### EQUAL RATES MODEL ###
#########################

#uses a single-rate model to reconstruct states at internal nodes; plots and reports rates
fitER<-ace(resp.syst,araneom_tree,model="ER",type="discrete")

#plots tree with pie charts with likelihood of ancestral states at nodes using the single rate model
pdf("ER-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red","yellow", "green", "blue"),levels(resp.syst))
tiplabels(pie=to.matrix(resp.syst.tips[araneom_tree$tip.label], levels(resp.syst.tips)),piecol=tipcols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=fitER$lik.anc,piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off() 

#############################
### SYMMETRIC RATES MODEL ###
#############################

#uses a model with symmetric rates between states to reconstruct states at internal nodes
fitSYM<-ace(resp.syst,araneom_tree,model="SYM",type="discrete")

#plots tree with pie charts with likelihood of ancestral states at nodes using the multiple rates model
pdf("SYM-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red","yellow", "green", "blue"),levels(resp.syst))
tiplabels(pie=to.matrix(resp.syst.tips[araneom_tree$tip.label], levels(resp.syst.tips)),piecol=tipcols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=fitSYM$lik.anc,piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off() 

##########################
### IRREVERSIBLE MODEL ###
##########################

#uses a model that disallows reversals from tracheae to booklungs, and transitions among tracheae
irreversible<-matrix(c(0,1,2,3,0,0,0,0,0,0,0,0,0,0,0,0),4,4,byrow=TRUE, dimnames=list(levels(resp.syst),levels(resp.syst)))
fitIRV<-ace(resp.syst,araneom_tree,model=irreversible,type="discrete")

#plots tree with pie charts with likelihood of ancestral states at nodes using the multiple rates model
pdf("IRV1-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red","yellow", "green", "blue"),levels(resp.syst))
tiplabels(pie=to.matrix(resp.syst.tips[araneom_tree$tip.label], levels(resp.syst.tips)),piecol=tipcols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=abs(fitIRV$lik.anc),piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off()

############################
### IRREVERSIBLE MODEL 2 ###
############################

#uses a model that disallows reversals from tracheae to booklungs
irreversible2<-matrix(c(0,1,2,3,0,0,4,5,0,6,0,7,0,8,9,0),4,4,byrow=TRUE, dimnames=list(levels(resp.syst),levels(resp.syst)))
fitIRV2<-ace(resp.syst,araneom_tree,model=irreversible2,type="discrete")

#plots tree with pie charts with likelihood of ancestral states at nodes using the multiple rates model
pdf("IRV2-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red","yellow", "green", "blue"),levels(resp.syst))
tiplabels(pie=to.matrix(resp.syst.tips[araneom_tree$tip.label], levels(resp.syst.tips)),piecol=tipcols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=abs(fitIRV2$lik.anc),piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off()

############################
### IRREVERSIBLE MODEL 3 ###
############################

#only prohibits lamella>booklungs and single_leaf>booklungs
irreversible3<-matrix(c(0,1,2,3,0,0,4,5,0,6,0,7,8,9,10,0),4,4,byrow=TRUE, dimnames=list(levels(resp.syst),levels(resp.syst)))
fitIRV3<-ace(resp.syst,araneom_tree,model=irreversible3,type="discrete")

#plots tree with pie charts with likelihood of ancestral states at nodes using the multiple rates model
pdf("IRV3-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red","yellow", "green", "blue"),levels(resp.syst))
tiplabels(pie=to.matrix(resp.syst.tips[araneom_tree$tip.label], levels(resp.syst.tips)),piecol=tipcols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=abs(fitIRV3$lik.anc),piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off()

#################################
### ALL RATES DIFFERENT MODEL ###
#################################

#uses a model allowing for different rates between states to reconstruct states at internal nodes
fitARD<-ace(resp.syst,araneom_tree,model="ARD",type="discrete")

#plots tree with pie charts with likelihood of ancestral states at nodes using the multiple rates model
pdf("ARD-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red","yellow", "green", "blue"),levels(resp.syst))
tiplabels(pie=to.matrix(resp.syst.tips[araneom_tree$tip.label], levels(resp.syst.tips)),piecol=tipcols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=fitARD$lik.anc,piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off() 

#######################
### TESTS MODEL FIT ###
#######################

#reports general results
sink(file = NULL)
sink("model_comparison.txt", append=FALSE, split=TRUE) 
print("model results:")
modelresults<-matrix(c(logLik(fitER),logLik(fitSYM),logLik(fitIRV),logLik(fitIRV2),logLik(fitIRV3),logLik(fitARD),length(fitER$rates),length(fitSYM$rates),length(fitIRV$rates),length(fitIRV2$rates),length(fitIRV3$rates),length(fitARD$rates)), nrow=2, ncol=6, byrow=TRUE, dimnames=list(c("log likelihood","number of parameters"),c("ER","SYM","IRV","IRV2","IRV3","ARD")))
modelresults
print(" ")
fitER
fitSYM
fitIRV
fitIRV2
fitIRV3
fitARD

#calculates fit of models using AIC...
AIC<-setNames(sapply(list(fitER,fitSYM,fitIRV,fitIRV2,fitIRV3,fitARD),AIC),c("ER","SYM","IRV","IRV2","IRV3","ARD"))
AIC_red<-setNames(sapply(list(fitER,fitSYM,fitIRV,fitARD),AIC),c("ER","SYM","IRV","ARD"))
print(" ")
print("AIC:")
AIC
print(" ")
print("AIC weights:")
aic.w(AIC)
aic.w(AIC_red)
print(" ")

#compares models using LRT...
print(" ")
print("LRT results:")
print("ER x SYM")
anova(fitER,fitSYM)
print("ER x IRV")
anova(fitER,fitIRV)
print("ER x IRV2")
anova(fitER,fitIRV2)
print("ER x IRV3")
anova(fitER,fitIRV3)
print("ER x ARD")
anova(fitER,fitARD)
print("SYM x ARD")
anova(fitSYM,fitARD)
print("IRV x SYM")
anova(fitIRV, fitSYM)
print("IRV x IRV2")
anova(fitIRV,fitIRV2)
print("IRV x IRV3")
anova(fitIRV,fitIRV3)
print("IRV2 x IRV3")
anova(fitIRV2,fitIRV3)
print("IRV x ARD")
anova(fitIRV,fitARD)
print("IRV2 x ARD")
anova(fitIRV2,fitARD)
print("IRV3 x ARD")
anova(fitIRV3,fitARD)
sink(file = NULL)