### Sequence-capture phylogenomics of early-diverging true spiders reveals convergent evolution of tracheal systems after the origin of aerial webs
### Martín J. Ramírez, Ivan L.F. Magalhaes, Shahan Derkarabetian, Joel Ledford, Charles E. Griswold, Hannah M. Wood, Marshal Hedin

###reconstruct ancestral states for posterior respiratory system morphology in spiders using different rate models
###tests fit of different models using AICs & Akaike weights

### coding scheme: foraging web coded as a multistate character: "aerial", "substrate", "webless"

#sets working directory and reads inputs, prunes tree to include only terminals listed in table with observations

require(phytools)
require(ape)

setwd("C:/Users/Usuario/Google Drive/Basal Araneomorphae/analyses morphology - Early Araneomoprhae/phytools/5d-Aerial_web_webless")
sink("output.txt", append=FALSE, split=TRUE) 

scorings<-read.csv("web_type.csv",row.names=1)
species<-rownames(scorings)
web.type<-setNames(scorings[,1],rownames(scorings))
web.type
unpruned_araneom_tree<-read.tree("araneomorphs.tre")
araneom_tree<-keep.tip(unpruned_araneom_tree, species)

#########################
### EQUAL RATES MODEL ###
#########################

#uses a single-rate model to reconstruct states at internal nodes; plots and reports rates
fitER<-ace(web.type,araneom_tree,model="ER",type="discrete")
ratesER<-fitMk(araneom_tree,web.type,model="ER")
pdf("ER-rates.pdf")
plot(ratesER)
dev.off() 
print("equal rates model:")
ratesER

#plots tree with pie charts with likelihood of ancestral states at nodes using the single rate model
pdf("ER-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red", "blue", "black"),levels(web.type))
tiplabels(pie=to.matrix(web.type[araneom_tree$tip.label], levels(web.type)),piecol=cols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=fitER$lik.anc,piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off() 

#############################
### SYMMETRIC RATES MODEL ###
#############################

#uses a model with symmetric rates between states to reconstruct states at internal nodes
fitSYM<-ace(web.type,araneom_tree,model="SYM",type="discrete")
ratesSYM<-fitMk(araneom_tree,web.type,model="SYM")
pdf("SYM-rates.pdf")
plot(ratesSYM)
dev.off() 
print("symmetric model:")
ratesSYM

#plots tree with pie charts with likelihood of ancestral states at nodes using the multiple rates model
pdf("SYM-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red", "blue", "black"),levels(web.type))
tiplabels(pie=to.matrix(web.type[araneom_tree$tip.label], levels(web.type)),piecol=cols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=fitSYM$lik.anc,piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off()

##########################
### IRREVERSIBLE MODEL ###
##########################

#uses a model that disallows regains of webs from webless spiders
irreversible<-model<-matrix(c(0,1,2,3,0,4,0,0,0),3,3,byrow=TRUE, dimnames=list(levels(web.type),levels(web.type)))
fitIRV<-ace(web.type,araneom_tree,model=irreversible,type="discrete")
ratesIRV<-fitMk(araneom_tree,web.type,model=irreversible)
pdf("IRV-rates.pdf")
plot(ratesIRV)
dev.off() 
print("irreversible model:")
ratesIRV

#plots tree with pie charts with likelihood of ancestral states at nodes using the multiple rates model
pdf("IRV-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red", "blue", "black"),levels(web.type))
tiplabels(pie=to.matrix(web.type[araneom_tree$tip.label], levels(web.type)),piecol=cols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=fitIRV$lik.anc,piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off() 

#################################
### ALL RATES DIFFERENT MODEL ###
#################################

#uses a model allowing for different rates between states to reconstruct states at internal nodes
fitARD<-ace(web.type,araneom_tree,model="ARD",type="discrete")
ratesARD<-fitMk(araneom_tree,web.type,model="ARD")
pdf("ARD-rates.pdf")
plot(ratesARD)
dev.off() 
print("all rates different model:")
ratesARD

#plots tree with pie charts with likelihood of ancestral states at nodes using the multiple rates model
pdf("ARD-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red", "blue", "black"),levels(web.type))
tiplabels(pie=to.matrix(web.type[araneom_tree$tip.label], levels(web.type)),piecol=cols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=fitARD$lik.anc,piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off() 

#######################
### TESTS MODEL FIT ###
#######################

#calculates fit of models using AIC...
AIC_rates<-setNames(sapply(list(ratesER,ratesSYM,ratesIRV,ratesARD),AIC),c("ER","SYM","IRV","ARD"))
AIC_fit<-setNames(sapply(list(fitER,fitSYM,fitIRV,fitARD),AIC),c("ER","SYM","IRV","ARD"))
sink(file = NULL)
sink("model_comparison.txt", append=FALSE, split=TRUE) 
print("AIC (rates):")
AIC_rates
print(" ")
print("AIC weights:")
aic.w(AIC_rates)
print(" ")

print("AIC (fit):")
AIC_fit
print(" ")
print("AIC weights:")
aic.w(AIC_fit)
print(" ")

#...or a LRT
kER<-length(ratesER$rates)
kSYM<-length(ratesSYM$rates)
kIRV<-length(ratesIRV$rates)
kARD<-length(ratesARD$rates)

modelresults<-matrix(c(logLik(ratesER), logLik(ratesSYM), logLik(ratesIRV), logLik(ratesARD), kER, kSYM, kIRV, kARD), nrow=2, ncol=4, byrow=TRUE, dimnames=list(c("log likelihood","number of parameters"),c("ER", "SYM", "IRV", "ARD")))
modelresults

LR_ERxSYM<--2*(logLik(ratesER)-logLik(ratesSYM))
LR_ERxARD<--2*(logLik(ratesER)-logLik(ratesARD))
LR_SYMxARD<--2*(logLik(ratesSYM)-logLik(ratesARD))
LR_IRVxARD<--2*(logLik(ratesIRV)-logLik(ratesARD))

P_chisq_ERxSYM<-pchisq(LR_ERxSYM,df=kSYM-kER)
P_chisq_ERxARD<-pchisq(LR_ERxARD,df=kARD-kER)
P_chisq_SYMxARD<-pchisq(LR_SYMxARD,df=kARD-kSYM)
P_chisq_IRVxARD<-pchisq(LR_IRVxARD,df=kARD-kIRV)
LRTresults<-matrix(c(LR_ERxSYM, LR_ERxARD, LR_SYMxARD, LR_IRVxARD, P_chisq_ERxSYM, P_chisq_ERxARD, P_chisq_SYMxARD, P_chisq_IRVxARD), nrow=2, ncol=4, byrow=TRUE, dimnames=list(c("LR","p"),c("ERxSYM", "ERxARD", "SYMxARD", "IRVxARD")))
print(" ")
print("LRT results:")
LRTresults
sink(file = NULL)