### Sequence-capture phylogenomics of early-diverging true spiders reveals convergent evolution of tracheal systems after the origin of aerial webs
### Martín J. Ramírez, Ivan L.F. Magalhaes, Shahan Derkarabetian, Joel Ledford, Charles E. Griswold, Hannah M. Wood, Marshal Hedin

###reconstruct ancestral states for posterior respiratory system morphology in spiders using different rate models
###tests fit of different models using AICs & Akaike weights

### coding scheme: presence of MAP spigots coded as "absent" or "present"

#sets working directory and reads inputs, prunes tree to include only terminals listed in table with observations

require(phytools)
require(ape)

#setwd("C:/Users/Usuario/Google Drive/Basal Araneomorphae/analyses morphology - Early Araneomoprhae/phytools/EDaraneomorphs/0-booklungs-nobooklungs")
setwd("C:/Users/ivanlfm/GoogleDrive/Basal Araneomorphae/analyses morphology - Early Araneomoprhae/phytools/exploring analysis/6a-silk_glands")
sink("output.txt", append=FALSE, split=TRUE) 

scorings<-read.csv("silk_glands.csv",row.names=1)
species<-rownames(scorings)
resp.syst<-setNames(scorings[,1],rownames(scorings))
resp.syst
unpruned_araneom_tree<-read.tree("araneomorphs.tre")
araneom_tree<-keep.tip(unpruned_araneom_tree, species)

#########################
### EQUAL RATES MODEL ###
#########################

#uses a single-rate model to reconstruct states at internal nodes; plots and reports rates
fitER<-ace(resp.syst,araneom_tree,model="ER",type="discrete")
ratesER<-fitMk(araneom_tree,resp.syst,model="ER")
pdf("ER-rates.pdf")
plot(ratesER)
dev.off() 
print("equal rates model:")
ratesER

#plots tree with pie charts with likelihood of ancestral states at nodes using the single rate model
pdf("ER-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red","black"),levels(resp.syst))
tiplabels(pie=to.matrix(resp.syst[araneom_tree$tip.label], levels(resp.syst)),piecol=cols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=fitER$lik.anc,piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off() 

##########################
### IRREVERSIBLE MODEL ###
##########################

#prohibits regains of MAP glands
irreversible<-matrix(c(0,0,1,0),2,2,byrow=TRUE, dimnames=list(levels(resp.syst),levels(resp.syst)))
fitIRV<-ace(resp.syst,araneom_tree,model=irreversible,type="discrete")
ratesIRV<-fitMk(araneom_tree,resp.syst,model=irreversible)
pdf("IRV-rates.pdf")
plot(ratesIRV)
dev.off() 
print("irreversible model:")
ratesIRV

#plots tree with pie charts with likelihood of ancestral states at nodes using the multiple rates model
pdf("IRV-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red","black"),levels(resp.syst))
tiplabels(pie=to.matrix(resp.syst[araneom_tree$tip.label], levels(resp.syst)),piecol=cols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=fitIRV$lik.anc,piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off()

#################################
### ALL RATES DIFFERENT MODEL ###
#################################

#uses a model allowing for different rates between states to reconstruct states at internal nodes
fitARD<-ace(resp.syst,araneom_tree,model="ARD",type="discrete")
ratesARD<-fitMk(araneom_tree,resp.syst,model="ARD")
pdf("ARD-rates.pdf")
plot(ratesARD)
dev.off() 
print("all rates different model:")
ratesARD

#plots tree with pie charts with likelihood of ancestral states at nodes using the multiple rates model
pdf("ARD-tree.pdf")
plotTree(ladderize(araneom_tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
cols<-setNames(c("red","black"),levels(resp.syst))
tiplabels(pie=to.matrix(resp.syst[araneom_tree$tip.label], levels(resp.syst)),piecol=cols,cex=0.3)
nodelabels(node=1:araneom_tree$Nnode+Ntip(araneom_tree), pie=fitARD$lik.anc,piecol=cols,cex=0.3)
add.simmap.legend(colors=cols,prompt=FALSE,x=0.9*par()$usr[1], y=0.8*par()$usr[3],fsize=0.8)
dev.off() 

#######################
### TESTS MODEL FIT ###
#######################

#calculates fit of models using AIC...
AIC<-setNames(sapply(list(ratesER, ratesIRV,ratesARD),AIC),c("ER","IRV","ARD"))
sink(file = NULL)
sink("model_comparison.txt", append=FALSE, split=TRUE) 
print("AIC:")
AIC
print(" ")
print("AIC weights:")
aic.w(AIC)
print(" ")

#...or a LRT
kER<-length(ratesER$rates)
kIRV<-length(ratesIRV$rates)
kARD<-length(ratesARD$rates)

modelresults<-matrix(c(logLik(ratesER), logLik(ratesIRV), logLik(ratesARD), kER, kIRV, kARD), nrow=2, ncol=3, byrow=TRUE, dimnames=list(c("log likelihood","number of parameters"),c("ER", "IRV", "ARD")))
modelresults

LR_ERxARD<--2*(logLik(ratesER)-logLik(ratesARD))
LR_IRVxARD<--2*(logLik(ratesIRV)-logLik(ratesARD))

P_chisq_ERxARD<-pchisq(LR_ERxARD,df=kARD-kER)
P_chisq_IRVxARD<-pchisq(LR_IRVxARD,df=kARD-kIRV)
LRTresults<-matrix(c(LR_ERxARD, LR_IRVxARD, P_chisq_ERxARD, P_chisq_IRVxARD), nrow=2, ncol=2, byrow=TRUE, dimnames=list(c("LR","p"),c("ERxARD", "IRVxARD")))
print(" ")
print("LRT results:")
LRTresults
sink(file = NULL)