### Sequence-capture phylogenomics of early-diverging true spiders reveals convergent evolution of tracheal systems after the origin of aerial webs
### Martín J. Ramírez, Ivan L.F. Magalhaes, Shahan Derkarabetian, Joel Ledford, Charles E. Griswold, Hannah M. Wood, Marshal Hedin

###reconstruct ancestral states for posterior respiratory system morphology in spiders using different models
###tests fit of different models using AICs & Akaike weights, and LRTs

### coding scheme: PRS coded as "booklungs", "lamellae", "single tube and leaf", "tubular tracheae", absences coded as an additional state
### reconstructs states in a preferred tree + a sample of trees from the posterior distribution of a Bayesian analysis

require(phytools)
require(ape)

setwd("C:/8resp-finely-coded-Bayesian")
sink("output.txt", append=FALSE, split=TRUE) 

##################
### TRAIT DATA ###
##################

#reads trait data, extracts species names and scorings
scorings<-read.csv("respsystem2.csv",row.names=1)
species<-rownames(scorings)
trait<-setNames(scorings[,1],rownames(scorings))

# sets up the irreversible custom model
#uses a model that disallows reversals from tracheae to booklungs, transitions among tracheae, and regain of respiratory structures from absence
irreversible<-matrix(c(0,0,0,0,0,1,0,2,3,4,5,0,0,0,0,6,0,0,0,0,7,0,0,0,0),5,5,byrow=TRUE, dimnames=list(levels(trait),levels(trait)))

#sets colors for each state (different for tips and nodes if needed, e.g. if there are inapplicables
cols<-setNames(c("black","red","yellow", "green", "blue"),levels(trait))
tipcols<-setNames(c("black","red","yellow", "green", "blue"),levels(trait))

#reads preferred tree
outgroup <- "Liphistius_sp_MIS1"
unpruned_preferred_tree <- read.tree("araneomorphs.tre")
preferred_tree <- ladderize(keep.tip(unpruned_preferred_tree, species), right = TRUE)
preferred_tree <- list(preferred_tree)
class(preferred_tree) <- "multiPhylo"

#reads trees from posterior distribution, selects 999 trees at random
ntopologies = 1000
posterior.trees <- read.tree("ExaBayes_topologies.run-0.myRun12.nwk", skip = 500) #skips 25% of the trees as burn-in
posterior.trees <- sample(posterior.trees, size = ntopologies-1)
class(posterior.trees) <- "multiPhylo"

#binds preferred tree to posterior trees
trees <- c(preferred_tree, posterior.trees)

############################################
### ANCESTRAL RECONSTRUCTION + MODEL FIT ###
############################################

AICcPost <- matrix(NA, nrow = ntopologies, ncol = 12)
colnames(AICcPost) <- c("ER-logLik", "SYM-logLik", "IRV-logLik", "ARD-logLik", "ER-AIC", "SYM-AIC","IRV-AIC","ARD-AIC", "ER-AICw", "SYM-AICw", "IRV-AICw", "ARD-AICw")
treeER <- "trees/treeER"
treeSYM <- "trees/treeSYM"
treeIRV <- "trees/treeIRV"
treeARD <- "trees/treeARD"
dir.create("trees")

for(i in 1:ntopologies) {

	#reroots trees, ladderizes, prunes tips without observations, resolves polytomies and zero-length branches, reorders scorings to match tip order
	tree <- trees[[i]]
		tree <- root(tree, outgroup)
		tree <- multi2di(tree, random = TRUE)
		tree$edge.length[tree$edge.length==0]<-1e-8
		tree <- ladderize(keep.tip(tree, species), right = TRUE)
		trees[[1]] <- tree
		data.order <- match(tree$tip.label, rownames(scorings))
		scorings.ordered = scorings[data.order, ]
		trait<-setNames(scorings.ordered,tree$tip.label)

	#ER model
		fitER<-ace(trait,tree,model="ER",type="discrete")
		filename <-  paste(treeER, i, ".pdf", sep = "")
		pdf(filename)
			plotTree(ladderize(tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
			tiplabels(pie=to.matrix(trait[tree$tip.label], levels(trait)),piecol=tipcols,cex=0.3)
			nodelabels(node=1:tree$Nnode+Ntip(tree), pie=fitER$lik.anc,piecol=cols,cex=0.3)
		dev.off()
		filename <-  paste(treeER, i, ".emf", sep = "")
		win.metafile(filename)
			plotTree(ladderize(tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
			tiplabels(pie=to.matrix(trait[tree$tip.label], levels(trait)),piecol=tipcols,cex=0.3)
			nodelabels(node=1:tree$Nnode+Ntip(tree), pie=fitER$lik.anc,piecol=cols,cex=0.3)
		dev.off()

	#SYM model
		fitSYM<-ace(trait,tree,model="SYM",type="discrete")
		filename <-  paste(treeSYM, i, ".pdf", sep = "")
		pdf(filename)
			plotTree(ladderize(tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
			tiplabels(pie=to.matrix(trait[tree$tip.label], levels(trait)),piecol=tipcols,cex=0.3)
			nodelabels(node=1:tree$Nnode+Ntip(tree), pie=fitSYM$lik.anc,piecol=cols,cex=0.3)
		dev.off()
		filename <-  paste(treeSYM, i, ".emf", sep = "")
		win.metafile(filename)
			plotTree(ladderize(tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
			tiplabels(pie=to.matrix(trait[tree$tip.label], levels(trait)),piecol=tipcols,cex=0.3)
			nodelabels(node=1:tree$Nnode+Ntip(tree), pie=fitSYM$lik.anc,piecol=cols,cex=0.3)
		dev.off()		
		
	#IRV model
		fitIRV<-ace(trait,tree,model=irreversible,type="discrete")
		filename <-  paste(treeIRV, i, ".pdf", sep = "")
		pdf(filename)
		plotTree(ladderize(tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
			tiplabels(pie=to.matrix(trait[tree$tip.label], levels(trait)),piecol=tipcols,cex=0.3)
			nodelabels(node=1:tree$Nnode+Ntip(tree), pie=abs(fitIRV$lik.anc),piecol=cols,cex=0.3)
		dev.off()
		filename <-  paste(treeIRV, i, ".emf", sep = "")
		win.metafile(filename)
		plotTree(ladderize(tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
			tiplabels(pie=to.matrix(trait[tree$tip.label], levels(trait)),piecol=tipcols,cex=0.3)
			nodelabels(node=1:tree$Nnode+Ntip(tree), pie=abs(fitIRV$lik.anc),piecol=cols,cex=0.3)
		dev.off()
	
	#ARD model
		fitARD<-ace(trait,tree,model="ARD",type="discrete")
		filename <-  paste(treeARD, i, ".pdf", sep = "")
		pdf(filename)
			plotTree(ladderize(tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
			tiplabels(pie=to.matrix(trait[tree$tip.label], levels(trait)),piecol=tipcols,cex=0.3)
			nodelabels(node=1:tree$Nnode+Ntip(tree), pie=fitARD$lik.anc,piecol=cols,cex=0.3)
		dev.off() 
		filename <-  paste(treeARD, i, ".emf", sep = "")
		win.metafile(filename)
			plotTree(ladderize(tree, right=TRUE),type="phylogram",fsize=0.5,ftype="b",lwd=1)
			tiplabels(pie=to.matrix(trait[tree$tip.label], levels(trait)),piecol=tipcols,cex=0.3)
			nodelabels(node=1:tree$Nnode+Ntip(tree), pie=fitARD$lik.anc,piecol=cols,cex=0.3)
		dev.off() 

	AIC<-sapply(list(fitER,fitSYM,fitIRV,fitARD),AIC)
	weights <- aic.w(AIC)

	AICcPost[i,] <-  c(logLik(fitER),logLik(fitSYM),logLik(fitIRV),logLik(fitARD), AIC[1], AIC[2], AIC[3], AIC[4], weights[1], weights[2], weights[3], weights[4])
}

write.tree(trees, "input_trees.nwk")
write.csv(AICcPost, "results.csv")
k<-setNames(c(length(fitER$rates), length(fitSYM$rates), length(fitIRV$rates), length(fitARD$rates)), c("ER", "SYM", "IRV", "ARD"))
write.csv(k, "parameters.csv")

sink(file = NULL)
#finished