### Sequence-capture phylogenomics of early-diverging true spiders reveals convergent evolution of tracheal systems after the origin of aerial webs
### Martín J. Ramírez, Ivan L.F. Magalhaes, Shahan Derkarabetian, Joel Ledford, Charles E. Griswold, Hannah M. Wood, Marshal Hedin

### tests the correlation between two discrete characters using the threshold model
# testing correlation of presence of MAP glands with presence of tracheae

require(phytools)
require(lattice)
require(coda)
setwd("C:/threshold_model/MAP_tracheae")
sink("output.txt", append=FALSE, split=TRUE) 

###########################
### LOADS TREE AND DATA ###
###########################

# Load your character matrix for the analysis. Characters have to be binary (0 and 1) so I converted the multistate character in presence of tracheae and presence of aerial web.
characters <- read.csv("Characters_coded_absent_present_FINAL_noMissing.csv",row.names=1)
species <-rownames(characters)
str(characters)
species

#Load the tree for the analysis, prunes taxa with missing data
unpruned_araneom_tree <- read.tree("araneomorphs.tre")
pagel_tree<-keep.tip(unpruned_araneom_tree, species)

#put your data in the same order as the tip order in pagel_tree phylogeny just to be safe
data.order <- match(pagel_tree$tip.label, rownames(characters)) #this is a vector ordered in the same way as taxa in the tree but with numbers representing order from data matrix

# Use that vector to rearrange our data matrix to match the tree:
characters.ordered = characters[data.order, ]
str(characters.ordered)

# pull out the two discrete characters of interest - the respirator system and presence of MAP. 
resp.syst<-setNames(characters.ordered[,2],rownames(characters.ordered))
map_gland<-setNames(characters.ordered[,5],rownames(characters.ordered))

#Combine the two characters
resp_and_map <- cbind(resp.syst, map_gland)
str(resp_and_map)

############
### MCMC ###
############

# set your parameters for the MCMC
sample <- 500
ngen <- 1000000 ##for real studies this number should be larger
burnin <- 0.2 * ngen

## Run analysis testing for correlation between two discrete characters.
mcmc <- threshBayes(pagel_tree,resp_and_map,types=c("disc","disc"), ngen=ngen, control=list(sample=sample, quiet=TRUE))

## Checks effective sample sizes
ESS <- mcmc$par[(burnin/sample + 1):nrow(mcmc$par), "r"]
class(ESS) <- "mcmc"
effectiveSize(ESS)
HPDinterval(ESS)

## plot our likelihood profile
win.metafile("likelihood-profile.emf")
plot(mcmc$par[,"logL"],type="l",xlab="generation",ylab="logL")
dev.off()

## plot our posterior sample for the correlation
win.metafile("density.emf")
plot(density(mcmc$par[(burnin/sample + 1):nrow(mcmc$par),"r"],bw=0.1),xlab="r", main="posterior density for r")
dev.off()

sink(file = NULL)

## what is the estimate of the correlation?

sink("correlation.txt", append=FALSE, split=TRUE)
print("correlation:")
mean(mcmc$par[(burnin/sample + 1):nrow(mcmc$par),"r"])
print("MCMC ESS:")
effectiveSize(ESS)
print("HPD interval:")
HPDinterval(ESS)
sink(file = NULL)
# Finished
